package betterwithmods.module.hardcore;

import betterwithmods.common.BWMRecipes;
import betterwithmods.common.BWOreDictionary;
import betterwithmods.module.Feature;
import betterwithmods.module.gameplay.CrucibleRecipes;
import com.google.common.collect.Sets;
import net.minecraft.init.Blocks;
import net.minecraft.init.Items;
import net.minecraft.item.ItemStack;
import net.minecraft.item.crafting.FurnaceRecipes;
import net.minecraft.item.crafting.IRecipe;
import net.minecraft.util.ResourceLocation;
import net.minecraftforge.fml.common.event.FMLInitializationEvent;
import net.minecraftforge.fml.common.event.FMLPostInitializationEvent;
import net.minecraftforge.oredict.OreDictionary;
import net.minecraftforge.oredict.ShapedOreRecipe;
import net.minecraftforge.oredict.ShapelessOreRecipe;
import org.apache.commons.lang3.tuple.Pair;

import java.util.Arrays;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * Created by tyler on 4/20/17.
 */
public class HCOres extends Feature {

    private static boolean oreNuggetSmelting, dustNuggetSmelting, fixVanillaRecipes;
    private static Set<String> oreExclude, dustExclude;
    private static int oreProductionCount, dustProductionCount;

    @Override
    public void setupConfig() {
        oreNuggetSmelting = loadPropBool("Ore to Nugget Smelting", "Make Ores (oredict ore.* )smelt into nuggets instead of ingots", true);

        oreExclude = Arrays.stream(loadPropStringList("Ore Exclude", "Oredictionary entries to exclude from ore to nugget smelting. Remove the prefix of the oredictionary. example 'oreIron' would be just 'iron' ", new String[0])).map(String::toLowerCase).collect(Collectors.toSet());
        dustExclude = Arrays.stream(loadPropStringList("Dust Exclude", "Oredictionary entries to exclude from dust to nugget smelting  Remove the prefix of the oredictionary. example 'dustIron' would be just 'iron'", new String[0])).map(String::toLowerCase).collect(Collectors.toSet());

        dustNuggetSmelting = loadPropBool("Dust to Nugget Smelting", "Make Dusts ( oredict dust.* ) smelt into nuggets instead of ingots", true);
        fixVanillaRecipes = loadPropBool("Fix Vanilla Recipes", "Make certain recipes cheaper to be more reasonable with nugget smelting, including Compass, Clock, and Bucket", true);

        oreProductionCount = loadPropInt("Ore Production Count", "Number of Materials returned from Smelting an Ore", 1);
        dustProductionCount = loadPropInt("Dust Production Count", "Number of Materials returned from Smelting a Dust", 1);
    }

    @Override
    public String getFeatureDescription() {
        return "Makes Ores only smelt into a single nugget, making it much harder to create large amounts of metal";
    }

    @Override
    public boolean requiresMinecraftRestartToEnable() {
        return true;
    }

    @Override
    public void init(FMLInitializationEvent event) {
        if (fixVanillaRecipes) {
            addHardcoreRecipe(new ShapedOreRecipe(null, Items.field_151111_aL, " N ", "NRN", " N ", 'N', "nuggetIron", 'R', "dustRedstone").setRegistryName(new ResourceLocation("minecraft", "compass")));
            addHardcoreRecipe(new ShapedOreRecipe(null, Items.field_151113_aN, " N ", "NQN", " N ", 'N', "nuggetGold", 'Q', "gemQuartz").setRegistryName(new ResourceLocation("minecraft", "clock")));
            addHardcoreRecipe(new ShapedOreRecipe(null, Items.field_151133_ar, "N N", " N ", 'N', "nuggetIron").setRegistryName(new ResourceLocation("minecraft", "bucket")));
            addHardcoreRecipe(new ShapelessOreRecipe(null, Items.field_151033_d, Items.field_151145_ak, "nuggetIron").setRegistryName(new ResourceLocation("minecraft", "flint_and_steel")));
        }
        CrucibleRecipes.addStokedCrucibleRecipe(new ItemStack(Items.field_191525_da, 3), new Object[]{new ItemStack(Items.field_151133_ar)});
        CrucibleRecipes.addStokedCrucibleRecipe(new ItemStack(Items.field_191525_da, 4), new Object[]{new ItemStack(Items.field_151111_aL)});
        CrucibleRecipes.addStokedCrucibleRecipe(new ItemStack(Items.field_151074_bl, 4), new Object[]{new ItemStack(Items.field_151113_aN)});
        CrucibleRecipes.addStokedCrucibleRecipe(new ItemStack(Items.field_191525_da), new Object[]{new ItemStack(Blocks.field_150479_bC, 2, 0)});
        CrucibleRecipes.addStokedCrucibleRecipe(new ItemStack(Items.field_191525_da), new Object[]{new ItemStack(Items.field_151033_d)});
    }

    private IRecipe addHardcoreRecipe(IRecipe recipe) {
        return registerHardcoreRecipe("HCOres", recipe);
    }

    @Override
    public void disabledInit(FMLInitializationEvent event) {
        CrucibleRecipes.addStokedCrucibleRecipe(new ItemStack(Items.field_151042_j, 3), new Object[]{new ItemStack(Items.field_151133_ar)});
        CrucibleRecipes.addStokedCrucibleRecipe(new ItemStack(Items.field_151042_j), new Object[]{new ItemStack(Blocks.field_150479_bC, 2, 0)});
        CrucibleRecipes.addStokedCrucibleRecipe(new ItemStack(Items.field_151042_j), new Object[]{new ItemStack(Items.field_151033_d, 1, OreDictionary.WILDCARD_VALUE)});
        CrucibleRecipes.addStokedCrucibleRecipe(new ItemStack(Items.field_151042_j, 4), new Object[]{new ItemStack(Items.field_151111_aL)});
        CrucibleRecipes.addStokedCrucibleRecipe(new ItemStack(Items.field_151043_k, 4), new Object[]{new ItemStack(Items.field_151113_aN)});
    }

    @Override
    public void postInit(FMLPostInitializationEvent event) {
        Set<String> oreExcludes = Sets.union(oreExclude, Sets.newHashSet("diamond"));
        if (oreNuggetSmelting) {
            List<Pair<ItemStack, String>> oreSuffixes = BWOreDictionary.oreNames.stream().map(i -> Pair.of(i, BWOreDictionary.getSuffix(i, "ore"))).filter(p -> !oreExcludes.contains(p.getValue().toLowerCase())).collect(Collectors.toList());
            oreSuffixes.forEach(pair -> OreDictionary.getOres("nugget" + pair.getValue()).stream().findFirst().ifPresent(nugget -> {
                ItemStack n = nugget.func_77946_l();
                n.func_190920_e(oreProductionCount);
                BWMRecipes.removeFurnaceRecipe(pair.getKey());
                FurnaceRecipes.func_77602_a().func_77599_b().put(pair.getKey(), n);
            }));
        }
        Set<String> dustExcludes = Sets.union(dustExclude, Sets.newHashSet("diamond"));
        if (dustNuggetSmelting) {
            List<Pair<ItemStack, String>> dustSuffixes = BWOreDictionary.dustNames.stream().map(i -> Pair.of(i, BWOreDictionary.getSuffix(i, "dust"))).filter( p -> p == null || p.getLeft() == null || p.getRight() == null).filter(p -> !dustExcludes.contains(p.getValue().toLowerCase())).collect(Collectors.toList());
            dustSuffixes.forEach(pair -> OreDictionary.getOres("nugget" + pair.getValue()).stream().findFirst().ifPresent(nugget -> {
                ItemStack n = nugget.func_77946_l();
                n.func_190920_e(dustProductionCount);
                BWMRecipes.removeFurnaceRecipe(pair.getKey());
                FurnaceRecipes.func_77602_a().func_77599_b().put(pair.getKey(), n);
            }));
        }
    }


}
